# charset = utf-8

from neomodel import (
    StructuredNode,
    StructuredRel,
    IntegerProperty,
    StringProperty,
    DateTimeProperty,
    RelationshipTo,
    FloatProperty
)


class BlockRel(StructuredRel):
    index = StringProperty(required=True)
    date = DateTimeProperty(required=True)


class Block(StructuredNode):
    hash = StringProperty(required=True, index=True)
    next_block = RelationshipTo('Block', 'NEXT_BLOCK', model=BlockRel)


class Miner(StructuredNode):
    address = StringProperty(required=True)
    pack = RelationshipTo('Block', 'PACK', model=BlockRel)


class Transaction(StructuredNode):
    hash = StringProperty(required=True, index=True)
    amount = FloatProperty(required=True)
    trade = RelationshipTo('Block', 'Trade', model=BlockRel)
    transfer_in = RelationshipTo('Account', 'TRANSFER_IN', model=BlockRel)
    invoke = RelationshipTo('Contract', 'INVOKE', model=BlockRel)


class Account(StructuredNode):
    address = StringProperty(required=True, index=True)
    transfer_out = RelationshipTo('Transaction', 'TRANSFER_OUT', model=BlockRel)


class Contract(StructuredNode):
    address = StringProperty(required=True, index=True)


def gen_node_json(node):
    if isinstance(node, Block):
        category, label = 1, "区块"
    elif isinstance(node, Transaction):
        category, label = 2, "交易"
    elif isinstance(node, Miner):
        category, label = 3, "矿工"
    elif isinstance(node, Account):
        category, label = 4, "账户"
    else:
        category, label = 5, "合约"
    return {
        "id": node.id,
        "category": category,
        "label": label,
        "properties": {}
    }


def gen_rel_json(r):
    s = r.start_node()
    e = r.end_node()
    if isinstance(s, Account):
        label = "转出"
    elif isinstance(s, Transaction):
        if isinstance(e, Account):
            label = '转入'
        elif isinstance(e, Contract):
            label = '调用'
        else:
            label = "交易区块"
    elif isinstance(s, Block):
        label = '下一区块'
    else:
        label = '打包'
    return {
        "id": r.id,
        "source": s.id,
        "target": e.id,
        "label": label
    }


def gen_node_with_properties_json(node, properties, max_id):
    res = gen_node_json(node)
    res["max_id"] = max_id
    res["properties"] = properties
    return res
