# charset = utf-8

from neomodel import (
    StructuredNode,
    StructuredRel,
    StringProperty,
    DateTimeProperty,
    RelationshipTo
)


class DomainRel(StructuredRel):
    index = StringProperty(required=True)
    date = DateTimeProperty(required=True)  # 请求时间


class RequestIp(StructuredNode):
    request_ip = StringProperty(required=True, unique_index=True)
    province = StringProperty(required=True, index=True)
    isp = StringProperty(required=True, index=True)
    request_domain = RelationshipTo('DomainName', 'REQUEST', model=DomainRel)


class DnsIp(StructuredNode):
    dns_ip = StringProperty(required=True, unique_index=True)
    parse_domain = RelationshipTo('DomainName', 'PARSE', model=DomainRel)


class DomainName(StructuredNode):
    domain_name = StringProperty(required=True, unique_index=True)
    label = RelationshipTo('Label', 'LABEL', model=DomainRel)


class Label(StructuredNode):
    label_name = StringProperty(required=True, unique_index=True)


def gen_node_json(node):
    if isinstance(node, RequestIp):
        category, label = 1, "请求IP"
    elif isinstance(node, DomainName):
        category, label = 2, "请求域名"
    elif isinstance(node, DnsIp):
        category, label = 3, "DNS服务器IP"
    else:
        category, label = 4, "标签"
    return {
        "id": node.id,
        "category": category,
        "label": label,
        "properties": {}
    }


def gen_node_json2(node):
    node_type = list(node.labels)[0]
    if node_type == "RequestIp":
        category, label = 1, "请求IP"
    elif node_type == "DomainName":
        category, label = 2, "请求域名"
    elif node_type == "DnsIp":
        category, label = 3, "DNS服务器IP"
    else:
        category, label = 4, "标签"
    return {
        "id": node.id,
        "category": category,
        "label": label,
        "properties": {}
    }


def gen_relation_json2(relation):
    r_type = relation.type
    if r_type == "REQUEST":
        label = "请求"
    elif r_type == "PARSE":
        label = "解析"
    else:
        label = "标签"
    return {
        "id": relation.id,
        "source": relation.nodes[0].id,
        "target": relation.nodes[1].id,
        "label": label
    }
