# charset = utf-8

from neomodel import (
    StructuredNode,
    StructuredRel,
    StringProperty,
    DateTimeProperty,
    RelationshipTo
)


class NextIpRel(StructuredRel):
    index = StringProperty(required=True)
    date = DateTimeProperty(required=True)


class RouteIp(StructuredNode):
    ip = StringProperty(required=True, unique_index=True)
    country = StringProperty(required=True)
    province = StringProperty(required=True, index=True)
    city = StringProperty(required=True, index=True)
    isp = StringProperty(required=True, index=True)
    start_ip = StringProperty(required=True, index=True)
    root_ip = StringProperty(required=True, index=True)
    next_ip = RelationshipTo('RouteIp', 'NEXT_IP', model=NextIpRel)


class FirstIp(RouteIp):
    pass


class EndIp(RouteIp):
    pass


def gen_node_json(node):
    if isinstance(node, FirstIp):
        category, label = 1, "起始路由"
    elif isinstance(node, EndIp):
        category, label = 3, "终止路由"
    else:
        category, label = 2, "中间路由"
    return {
        "id": node.id,
        "category": category,
        "label": label,
        "properties": {
            "ip": node.ip,
            "province": node.province,
            "city": node.city,
            "isp": node.isp,
        }
    }


def gen_rel_json(r):
    s = r.start_node()
    e = r.end_node()
    if isinstance(s, FirstIp):
        label = "第一跳"
    elif isinstance(e, EndIp):
        label = '最后一跳'
    else:
        label = '下一跳'
    return {
        "id": r.id,
        "source": s.id,
        "target": e.id,
        "label": label,
    }
